﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class BlueCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};
            float gf;
            float gbb;
            float gd;

            0 => {0};
            0 => gf;
            0 => gbb;
            0 => gd;

            class Shimmer 
            {{
                // universal gain;
                float g;
                1 => g;
                
                // 1st oscillator collection: triangle tone
                // initialize oscillators for audio output
                TriOsc shim0L => Gain g0L => dac;
                TriOsc shim1L => Gain g1L => dac;
                TriOsc shim0R => Gain g0R => dac;
                TriOsc shim1R => Gain g1R => dac;
                
                // initialize oscillators for amplitude
                SinOsc a0L => blackhole;
                SinOsc a1L => blackhole;
                SinOsc a0R => blackhole;
                SinOsc a1R => blackhole;
                
                // set amplitude frequencies
                Math.random2f(3.0, 5.0) => float aL;
                aL => a0L.freq;
                aL => a1L.freq;
                Math.random2f(0.5, 1.0) => float aR;
                aR => a0R.freq;
                aR => a1R.freq;
                
                
                // 2nd oscillator collection: sine tone
                // initialize oscillators for audio output
                SinOsc shim2L => Gain g2L => dac;
                SinOsc shim3L => Gain g3L => dac;
                SinOsc shim2R => Gain g2R => dac;
                SinOsc shim3R => Gain g3R => dac;
                
                // initialize oscillators for amplitude
                SinOsc a2L => blackhole;
                SinOsc a3L => blackhole;
                SinOsc a2R => blackhole;
                SinOsc a3R => blackhole;
                
                // set amplitude frequencies
                Math.random2f(1.0, 2.0) => aL;
                aL => a2L.freq;
                aL => a3L.freq;
                Math.random2f(2.0, 3.0) => aR;
                aR => a2R.freq;
                aR => a3R.freq;
                
                
                // 3rd oscillator collection: panning sine tone component
                // initialize oscillator for audio output
                SinOsc shim4 => Gain g4 => Pan2 p => dac;
                
                // initialize oscillators for amplitude
                SinOsc a4 => blackhole;
                SinOsc a5 => blackhole;
                
                // set amplitude frequencies
                0.2 => a4.freq;
                Math.random2f(5.0, 10.0) => a5.freq;
                
                // initialize, set up panning
                SinOsc p4 => blackhole;
                0.5 => p4.freq;
                
                
                fun void Set_F0(float f0)
                {{
                    f0 => float freq0;
                    freq0 * 1.5 => shim0L.freq;
                    freq0 * 4 => shim1L.freq;
                    freq0 * 3.5 => shim0R.freq;
                    freq0 * 12 => shim1R.freq;
                    
                    f0 * 5/4 => float freq2;
                    freq2 * 1.5 => shim2L.freq;
                    freq2 * 2.5 => shim3L.freq;
                    freq2 * 1 => shim2R.freq;
                    freq2 * 23 => shim3R.freq;
                    
                    freq2 * 3.7 => float freq4;
                    freq4 => shim4.freq;
                }}
                
                
                fun void Update_Osc(int num)
                {{
                    if (num==1)
                    {{
                        Math.sin(a0L.phase() * 2 * pi) * 0.1 * g => g0L.gain;
                        Math.sin(a1L.phase() * 2 * pi) * 0.1 * g => g1L.gain;
                        Math.sin(a0R.phase() * 2 * pi) * 0.1 * g => g0R.gain;
                        Math.sin(a1R.phase() * 2 * pi) * 0.1 * g => g1R.gain;
                    }}
                    else if (num == 2)
                    {{
                        Math.sin(a2L.phase() * 2 * pi) * 0.1 * g => g2L.gain;
                        Math.sin(a3L.phase() * 2 * pi) * 0.1 * g => g3L.gain;
                        Math.sin(a2R.phase() * 2 * pi) * 0.1 * g => g2R.gain;
                        Math.sin(a3R.phase() * 2 * pi) * 0.1 * g => g3R.gain;
                    }}
                    else if (num == 3)
                    {{
                        Math.sin(p4.phase() * 2 * pi) => p.pan;
                        Math.sin(a4.phase() * 2 * pi) * Math.sin(a5.phase() * 2 * pi) * 0.2 => g4.gain;
                    }}
                }}
                
                
                fun void Set_Gain(float ug)
                {{
                    ug * 0.3 => g;
                }}
            }} 
                
                
            Shimmer F;
            F.Set_F0(349.2);
            F.Set_Gain(gf);

            Shimmer Bb;
            Bb.Set_F0(466.2);
            Bb.Set_Gain(gbb);

            Shimmer D;
            D.Set_F0(587.3);
            D.Set_Gain(gd);


            // play all collections
            while(true)
            {{
                if({0} == 0)
                {{
                    if(gf < 0.5)
                    {{
                        0.00001 +=> gf;
                        F.Set_Gain(gf);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(gbb < 0.6)
                    {{
                        0.00001 +=> gbb;
                        Bb.Set_Gain(gbb);
                    }}
                    else
                    {{
                        1 +=> {0};
                        
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(gd < 0.5)
                    {{
                        0.00001 +=> gd;
                        D.Set_Gain(gd);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 3)
                {{
                    if(gbb > 0.1)
                    {{
                        0.0001 -=> gf;
                        0.0001 -=> gbb;
                        0.0001 -=> gd;
                        
                        F.Set_Gain(gf);
                        Bb.Set_Gain(gbb);
                        D.Set_Gain(gd);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                // update oscillators
                F.Update_Osc(1);
                F.Update_Osc(2);
                F.Update_Osc(3);
                
                Bb.Update_Osc(1);
                Bb.Update_Osc(2);
                Bb.Update_Osc(3);
                
                D.Update_Osc(1);
                D.Update_Osc(2);
                D.Update_Osc(3);


                // advance time
                1::ms => now;
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
