﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class GreenCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};

            float g;
            float max_pick;
            float min_pick;
            float max_dur;

            0 => {0};
            0.2 => g;
            50 => max_pick;
            Math.round(3 * max_pick / 4) => min_pick;
            500 => max_dur;

            // our patch
            Shakers shake => NRev r => dac;
            0.2 => r.mix;
            g => dac.gain;

            // our main loop
            while( true )
            {{
                if({0} == 0)
                {{
                    if(g < 2)
                    {{
                        0.05 +=> g; // CHECK THIS VALUE
                        g => dac.gain;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(max_pick > 5)
                    {{
                        1 -=> max_pick;
                        Math.round(max_pick / 2) => min_pick;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(max_dur > 200)
                    {{
                        5 -=> max_dur;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 3)
                {{
                    if(g > 0.5)
                    {{
                        0.03 -=> g;
                        g => dac.gain;
                    }}
                    else
                    {{ 
                        1 +=> {0};
                    }}
                }}
                
                // frequency..
                if( Math.randomf() > 0.625 )
                {{
                    Math.random2( 13, 19 ) => shake.which;
                    Std.mtof( Math.random2f( 0.0, 128.0 ) ) => shake.freq;
                    Math.random2f( 1, 128 ) => shake.objects;
                }}
                
                // shake it!
                Math.random2f(0.8, 1.3) => shake.noteOn;
                if(Math.randomf() > 0.9)
                {{
                    max_dur::ms => now; 
                }}
                else if(Math.randomf() > .7)
                {{ 
                    Math.round(max_dur / 2)::ms => now; 
                }}
                else if(Math.randomf() > .3)
                {{
                    Math.round(max_dur / 3)::ms => now;
                }}
                else if(Math.randomf() > .1)
                {{ 
                    Math.round(max_dur / 4)::ms => now; 
                }}
                else
                {{
                    1 => int i => int pick_dir;
                    4 * Math.random2f(min_pick, max_pick) => float pick;
                    0.0 => float pluck;
                    0.7 / pick => float inc;
                    
                    for( ; i < pick; i++ )
                    {{
                        75::ms => now;
                        Math.random2f(.2,.3) + i*inc => pluck;
                        pluck + -.2 * pick_dir => shake.noteOn;
                        !pick_dir => pick_dir;
                    }}
                }}
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
