﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class OrangeCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};
            float q;
            int max_dur;
            int min_dur;

            0 => {0};
            20 => q;
            300 => max_dur;
            200 => min_dur;

            SawOsc sl => BPF bl => Gain ol => dac;
            SawOsc sr => BPF br => Gain or => dac;

            ol => Delay dl => ol;
            or => Delay dr => or;

            0.5 => sl.gain;
            0.5 => sr.gain;

            q => bl.Q;
            q => br.Q;

            // our radius
            .999 => float R;
            // our delay order
            4000 => float L;

            // set delay
            L::samp => dl.delay;
            L::samp => dr.delay;

            // set dissipation factor
            Math.pow(R, L) => dl.gain;
            Math.pow(R, L) => dr.gain;

            while(true)
            {{
                if({0} == 0)
                {{
                    if(max_dur > 20)
                    {{
                        1 -=> max_dur;
                        
                        if(min_dur > 10)
                        {{
                            1 -=> min_dur;
                        }}
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(L > 50)
                    {{
                        10 -=> L;
                        Math.pow(R, L) => dl.gain;
                        Math.pow(R, L) => dr.gain;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(q < 70)
                    {{
                        0.1 +=> q;
                        q => bl.Q;
                        q => br.Q;
                    }}
                    else
                    {{
                        1 +=> {0};
                        10 => max_dur;
                        1 => min_dur;
                    }}
                }}
                
                else if({0} == 3)
                {{
                    if(L < 4000)
                    {{
                        10 +=> L;
                        Math.pow(R, L) => dl.gain;
                        Math.pow(R, L) => dr.gain;
                    }}
                    else if(q > 50)
                    {{
                        1 -=> q;
                        q => bl.Q;
                        q => br.Q;
                    }}
                    else
                    {{
                        1 +=> {0};
                        0.9 => R;
                        Math.pow(R, L) => dl.gain;
                        Math.pow(R, L) => dr.gain;
                        0.05 => sl.gain;
                        0.05 => sr.gain;
                    }}
                }}
                
                Math.pow(2, Math.randomf() * 4) * 300 => bl.freq;
                Math.pow(2, Math.randomf() * 4) * 300 => br.freq;

                Math.random2f(100, 300) => sl.freq;
                Math.random2f(100, 300) => sr.freq;
                Math.random2(min_dur, max_dur)::ms => now;
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
