﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class PinkCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};
            float g;
            float dur_wiggles;

            0 => {0};
            0.01 => g;
            1.0 => dur_wiggles;


            //// //// //// //// //// 
            // CLASS CONSTRUCTION
            //// //// //// //// ////

            // class for single frequency tone with frequency wiggle feature
            class Harmonic
            {{
                // construct tone and send out
                SinOsc sine; Gain gain; NRev rev;
                sine => gain => rev => dac; 
                
                // specify mix
                0.5 => rev.mix;
                
                // function to set tone frequency
                fun void set_freq( float f )
                {{
                    f => sine.freq;
                }}
                
                // function to set tone gain
                fun void set_gain( float g )
                {{
                    g => gain.gain;
                }}

                // function to wiggle frequency within a window
                fun void wiggle( SinOsc sine, float f, float win )
                {{
                    Math.random2f( -win/2, win/2 ) => float wiggle;
                    wiggle +=> f;
                    f => sine.freq;
                }}
            }}


            // class for five-frequency complex tone built of Harmonic objects
            class Note
            {{
                // initialize parameters
                float f0;
                float vol;
                int sub;
                
                // initialize Harmonics
                Harmonic f0x1;
                Harmonic f0x2;
                Harmonic f0x3;
                Harmonic f0x8;
                Harmonic f0sub;
                
                // function to set fundamental frequency
                fun void set_f0( float f )
                {{
                    midi2freq( f ) => f0;
                    
                    Math.random2( 0, 3 ) => sub;
                    
                    // pre-determined harmonic structure of Note
                    [1*f0, 2*f0, 3*f0, 8*f0] @=> float freqs[];
                    
                    f0x1.set_freq( freqs[0] );
                    f0x2.set_freq( freqs[1] );
                    f0x3.set_freq( freqs[2] );
                    f0x8.set_freq( freqs[3] );
                    
                    f0sub.set_freq( freqs[sub]/2 ); 
                }}
                
                // function to set volume
                fun void set_vol( float v )
                {{
                    v => vol;
                    
                    // pre-determined level balace of Note
                    [0.3*vol, 0.15*vol, 0.1*vol, 0.03*vol] @=> float gains[];
                    
                    f0x1.set_gain( gains[0] );
                    f0x2.set_gain( gains[1] );
                    f0x3.set_gain( gains[2] );
                    f0x8.set_gain( gains[3] );
                    
                    f0sub.set_gain( gains[0] ); 
                }}
                
                // function to convert midi notes to Hz
                fun float midi2freq( float m )
                {{
                    Math.pow(2, (m - 69) / 12) * 440 => float f;
                    return f;
                }}

                // function to wiggle Harmonics
                fun void wiggle()
                {{
                    2 * 3 * Math.ceil( f0/100 )  => float win;
                    
                    f0x1.wiggle( f0x1.sine, f0x1.sine.freq(), win );
                    f0x2.wiggle( f0x2.sine, f0x2.sine.freq(), win );
                    f0x3.wiggle( f0x3.sine, f0x3.sine.freq(), win );
                    f0x8.wiggle( f0x8.sine, f0x8.sine.freq(), win );
                    
                    f0sub.wiggle( f0sub.sine, f0sub.sine.freq(), win );
                }}
            }}


            //// //// //// ////
            // SOUND SYNTHESIS
            //// //// //// ////

            Note note;
            note.set_vol(g);
                
            note.set_f0( 300 );
                
            while( true )
            {{
                note.wiggle();
                dur_wiggles::second => now;
                
                if({0} == 0)
                {{
                    if(g < 0.15)
                    {{
                        0.01 +=> g;
                        note.set_vol(g);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(dur_wiggles > 0.1)
                    {{
                        1.05 /=> dur_wiggles;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(dur_wiggles > 0.05)
                    {{
                        1.01 /=> dur_wiggles;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                else if({0} == 3)
                {{
                    if(dur_wiggles < 0.2)
                    {{
                        1.01 *=> dur_wiggles;
                        if(g > 0.05)
                        {{
                            0.05 -=> g;
                        }}
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        //ps = transform.GetComponent<ParticleSystem>();
        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // fades
        else if(evolution == 4)
        {
            //_size += 0.005f;
            //halo.FindProperty("m_Size").floatValue = _size;
            //halo.ApplyModifiedProperties();

            //ps_on = Random.Range(0, 5);
            //if (ps_on == 1)
            //{
            //    ps.Emit(1);
            //}
            ps.Stop();
        }
        // chillin'
        else if(evolution == 5)
        {
            ps.Stop();
        }
    }
}
