﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class RedCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};
            float g;
            float del;
            float cf;
            float mg;

            0 => {0};
            0.05 => g;
            15 => del;
            220 => cf;
            200 => mg;


            SinOsc m => SinOsc c => Gain g1 => DelayL d => dac;
            m => c => Gain g2 => dac;
                
            // carrier frequency
            cf => c.freq;
            // modulator frequency
            20 => m.freq;
            // index of modulation
            mg => m.gain;
                
            //2 => c.sync;
                
            // feedback
            d => Gain g3 => d;
                
            // set parameters
            del::ms => d.delay;
            g => g1.gain;
            g => g2.gain;
            0.95 => g3.gain;


            while( true ) 
            {{
                if({0} == 0)
                {{
                    if(g < 0.2)
                    {{
                        0.0005 +=> g;
                        g => g1.gain;
                        g => g2.gain;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(del < 50)
                    {{
                        0.5 +=> del;
                        del::ms => d.delay;
                    }}
                    else
                    {{
                        1 +=> {0};
                        
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(cf < 1000)
                    {{
                        10 +=> cf;
                        cf => c.freq;
                    }}
                    else if(mg < 500)
                    {{
                        2 +=> mg;
                        mg => m.gain;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }} 
                }}
                
                else if({0} == 3)
                {{
                    if(g > 0.05)
                    {{
                        0.005 -=> g;
                        g => g1.gain;
                        g => g2.gain;
                    }}
                    else if(mg > 300)
                    {{
                        2 -=> mg;
                        mg => m.gain;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                Math.random2f(0.5, 10.0) * 20 => float f;
                f => m.freq;
                
                100::ms => now;
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
