﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class VioletCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};
            0 => {0};

            class Sweep
            {{
                TriOsc s => LPF l => ResonZ q => NRev r => dac;
                
                // set parameters
                float freq_save;
                440 => freq_save;
                freq_save => s.freq;
                
                3000 => l.freq;
                
                1000 => q.freq;
                50 => q.Q;
                
                0.5 => r.mix;
                
                
                fun void set_sin(float f, float g)
                {{
                    f => freq_save;
                    freq_save => s.freq;
                    g => s.gain;
                }}
                
                fun void set_lpf(float f)
                {{
                    f => l.freq;
                }}
                
                fun void set_res(float f, float Q)
                {{
                    f => q.freq;
                    Q => q.Q;
                }}
                
                fun void set_rev(float m)
                {{
                    m => r.mix;
                }}    
                
                fun void wiggle()
                {{
                    3 * Math.ceil( freq_save/100 )  => float win;
                    Math.random2f( -win/2, win/2 ) => float wiggle;
                    wiggle +=> freq_save;
                    freq_save => s.freq;
                }}
            }}


            Sweep sweep1;
            Sweep sweep2;
            Sweep sweep3;
            Sweep sweep4;
            Sweep sweep5;

            100 => float f0;
            0.5 => float g0;
            0 => int i;

            sweep1.set_sin(f0, g0);
            sweep2.set_sin(2*f0, 13/16*g0);
            sweep3.set_sin(3*f0, 5/8*g0);
            sweep4.set_sin(4*f0, 1/2*g0);
            sweep5.set_sin(5*f0, 3/8*g0);

            float t;
            50 => float move;
            while( true )
            {{
                // sweep the cutoff
                10 + Std.fabs(Math.sin(t)) * 1000 => float f;
                sweep1.set_res(f, 10);
                sweep2.set_res(1.3*f, 40);
                sweep3.set_res(1.5*f, 20);
                sweep4.set_res(2*f, 30);
                sweep5.set_res(2.7*f, 50);
                
                sweep1.wiggle();
                sweep2.wiggle();
                sweep3.wiggle();
                sweep4.wiggle();
                sweep5.wiggle();
                
                if({0} == 0)
                {{
                    if(g0 < 2)
                    {{
                        0.003 +=> g0;
                        
                        sweep1.set_sin(f0, g0);
                        sweep2.set_sin(2*f0, 13/16*g0);
                        sweep3.set_sin(3*f0, 5/8*g0);
                        sweep4.set_sin(4*f0, 1/2*g0);
                        sweep5.set_sin(5*f0, 3/8*g0);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 1)
                {{
                    if(move > 20)
                    {{
                        0.1 -=> move;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 2)
                {{
                    if(i < 1000)
                    {{
                        Math.random2f(-10.0, 10.0) +=> f0;
                        if(f0<0)
                        {{
                            -1 *=> f0;
                        }}
                        
                        sweep1.set_sin(f0, g0);
                        sweep2.set_sin(2*f0, 13/16*g0);
                        sweep3.set_sin(3*f0, 5/8*g0);
                        sweep4.set_sin(4*f0, 1/2*g0);
                        sweep5.set_sin(5*f0, 3/8*g0);
                        
                        1 +=> i;
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                else if({0} == 3)
                {{
                    if(g0 > 0.5)
                    {{
                        0.0005 -=> g0;
                        
                        sweep1.set_sin(f0, g0);
                        sweep2.set_sin(2*f0, 13/16*g0);
                        sweep3.set_sin(3*f0, 5/8*g0);
                        sweep4.set_sin(4*f0, 1/2*g0);
                        sweep5.set_sin(5*f0, 3/8*g0);
                    }}
                    else
                    {{
                        1 +=> {0};
                    }}
                }}
                
                .005 +=> t;
                move::ms => now;
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
