﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

public class YellowCrumb : MonoBehaviour
{
    public int evolution = 0;
    public string state_str;
    float inc;

    ParticleSystem ps;
    int ps_on;

    [SerializeField] private bool _isEnabled = true;
    [SerializeField] private float _size = 0.35f;
    SerializedObject halo;

    Color alphaColor;
    float timeToFade = 2.0f;

    ChuckSubInstance myChuck;
    ChuckIntSyncer state;


    void Start()
    {
        state_str = string.Concat("state", PlayerController.count.ToString());

        myChuck = GetComponent<ChuckSubInstance>();
        myChuck.RunCode(string.Format(@"
            global int {0};

            class Dinky
            {{
                // impulse to filter to dac
                Impulse i => BiQuad f => Envelope e;
                // set the filter's pole radius
                .99 => f.prad;
                // set equal gain zeros
                1 => f.eqzs;
                // set filter gain
                .2 => f.gain;
                // set the envelope
                .004::second => e.duration;
                
                public void radius( float rad )
                {{ rad => f.prad; }}
                
                public void gain( float g )
                {{ g => i.gain; }}
                
                public void connect( UGen ugen )
                {{ e => ugen; }}
                
                // t is for trigger
                public void t( float freq )
                {{
                    // set the current sample/impulse
                    1.0 => i.next;
                    // set filter resonant frequency
                    freq => f.pfreq;
                    // open the envelope
                    e.keyOn();
                }}
                
                // t is for trigger (using MIDI notes)
                public void t( int note )
                {{ t( Std.mtof( note ) ); }}
                
                // another lazy name: c (for close)
                public void c() {{ e.keyOff(); }}
            }}


            fun void Dink_go(int hi[], int bot, int top, float rm, float lev)
            {{
                // instantiate a Dinky (not connected yet)
                Dinky imp;
                
                // connect the rest of the patch
                Gain g => NRev r => Echo e => Echo e2 => dac;
                // direct/dry
                g => dac;
                e => dac;
                
                // set up delay, gain, and mix
                1500::ms => e.max => e.delay;
                3000::ms => e2.max => e2.delay;
                lev => g.gain;
                0.5 => e.gain;
                0.25 => e2.gain;
                rm => r.mix;
                
                // connect the Dinky
                // (in a future version of chuck, Dinky can be defined as an UGen)
                imp.connect( g );
                // set the radius (should never be more than 1)
                imp.radius( .999 );
                
                while(true)
                {{
                    // trigger
                    55 + Math.random2(bot, top) * 12 + hi[Math.random2(0,hi.cap()-1)] => imp.t;
                    // let time pass
                    395::ms => now;
                    // close the envelope
                    imp.c();
                    // let a bit more time pass
                    50::ms => now;
                }}

            }}

            0 => {0};
            spork ~Dink_go([0, 2, 4, 5, 7, 9, 11], 2, 4, 0, 1);
            15::second => now;

            1 => {0};
            spork ~Dink_go([0, 2, 4, 6, 7, 9, 11], 2, 4, 0.3, 1);
            15::second => now;

            2 => {0};
            spork ~Dink_go([1, 2, 4, 5, 7, 9, 11], 1, 3, 0.1, 2);
            15::second => now;

            3 => {0};
            spork ~Dink_go([0, 2, 4, 5, 8, 9, 11], -1, 1, 0.2, 1.5);
            15::second => now;

            4 => {0};

            while( true )
            {{
                1::second => now;
            }}
        ", state_str));

        state = gameObject.AddComponent<ChuckIntSyncer>();
        state.SyncInt(myChuck, state_str);
        state.SetNewValue(0);

        // set direction and speed of rotation
        inc = Random.Range(-1.0f, 1.0f);

        ps = transform.GetChild(0).GetComponent<ParticleSystem>();
        ps.Stop();

        halo = new SerializedObject(transform.GetComponent("Halo"));

        alphaColor = transform.GetComponent<MeshRenderer>().material.color;
        alphaColor.a = 0;

        PlayerController.count += 1;
    }


    void Update()
    {
        evolution = state.GetCurrentValue();
        transform.Rotate(0f, inc, 0f, Space.Self);

        // drop crumb
        if(evolution == 0)
        {

        }
        // crumb, halo grow
        else if (evolution == 1)
        {
            transform.localScale += new Vector3(0.001f, 0.001f, 0.001f);

            _size += 0.002f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();
        }
        // crumb fades, halo chills
        else if(evolution == 2)
        {
            transform.GetComponent<MeshRenderer>().material.color = Color.Lerp(transform.GetComponent<MeshRenderer>().material.color, alphaColor, timeToFade * Time.deltaTime);
        }
        // halo shatters
        else if(evolution == 3)
        {
            _size += 0.005f;
            halo.FindProperty("m_Size").floatValue = _size;
            halo.ApplyModifiedProperties();

            ps_on = Random.Range(0, 5);
            //ps_on = 1;
            if(ps_on == 1)
            {
                ps.Emit(1);
            }
        }
        // chillin'
        else if(evolution == 4)
        {
            ps.Stop();
        }
    }
}
